@@ -91,9 +91,10 @@ enum {
 static guint signals[SIGNAL_LAST] = { 0 };
 
 typedef enum {
-    TRANSACTION_TYPE_HOST  = 0,
-    TRANSACTION_TYPE_MODEM = 1,
-    TRANSACTION_TYPE_LAST  = 2
+    TRANSACTION_TYPE_UNKNOWN = -1,
+    TRANSACTION_TYPE_HOST    = 0,
+    TRANSACTION_TYPE_MODEM   = 1,
+    TRANSACTION_TYPE_LAST    = 2
 } TransactionType;
 
 typedef enum {
@@ -258,6 +259,8 @@ device_release_transaction (MbimDevice      *self,
     GTask              *task;
     TransactionContext *ctx;
 
+    g_assert ((type != TRANSACTION_TYPE_UNKNOWN) && (type < TRANSACTION_TYPE_LAST));
+
     /* Only return transaction if it was released from the HT */
     if (!self->priv->transactions[type])
         return NULL;
@@ -356,6 +359,8 @@ device_store_transaction (MbimDevice       *self,
 {
     TransactionContext *ctx;
 
+    g_assert ((type != TRANSACTION_TYPE_UNKNOWN) && (type < TRANSACTION_TYPE_LAST));
+
     transaction_task_trace (task, "store");
 
     if (G_UNLIKELY (!self->priv->transactions[type]))
@@ -579,11 +584,13 @@ process_message (MbimDevice        *self,
         GError             *error = NULL;
         GTask              *task;
         TransactionContext *ctx;
+        TransactionType     transaction_type = TRANSACTION_TYPE_UNKNOWN;
 
         if (MBIM_MESSAGE_GET_MESSAGE_TYPE (message) == MBIM_MESSAGE_TYPE_INDICATE_STATUS) {
             /* Grab transaction */
+            transaction_type = TRANSACTION_TYPE_MODEM;
             task = device_release_transaction (self,
-                                               TRANSACTION_TYPE_MODEM,
+                                               transaction_type,
                                                MBIM_MESSAGE_TYPE_INDICATE_STATUS,
                                                mbim_message_get_transaction_id (message));
 
@@ -598,8 +605,9 @@ process_message (MbimDevice        *self,
         } else {
             /* Grab transaction. This is a _DONE message, so look for the request
              * that generated the _DONE */
+            transaction_type = TRANSACTION_TYPE_HOST;
             task = device_release_transaction (self,
-                                               TRANSACTION_TYPE_HOST,
+                                               transaction_type,
                                                (MBIM_MESSAGE_GET_MESSAGE_TYPE (message) - 0x80000000),
                                                mbim_message_get_transaction_id (message));
             if (!task) {
@@ -669,7 +677,7 @@ process_message (MbimDevice        *self,
 
         /* Need more fragments, store transaction */
         g_assert (device_store_transaction (self,
-                                            TRANSACTION_TYPE_HOST,
+                                            transaction_type,
                                             task,
                                             MAX_TIME_BETWEEN_FRAGMENTS_MS,
                                             NULL));
@@ -862,8 +870,9 @@ get_descriptors_filepath (MbimDevice *self)
     GUdevDevice *device = NULL;
     GUdevDevice *parent_device = NULL;
     GUdevDevice *grandparent_device = NULL;
-    gchar *descriptors_path = NULL;
-    gchar *device_basename = NULL;
+    gchar       *descriptors_path = NULL;
+    gchar       *device_basename = NULL;
+    GError      *error = NULL;
 
     client = g_udev_client_new (NULL);
     if (!G_UDEV_IS_CLIENT (client)) {
@@ -884,7 +893,14 @@ get_descriptors_filepath (MbimDevice *self)
      *   Which is the one with the descriptors file.
      */
 
-    device_basename = g_path_get_basename (self->priv->path);
+    device_basename = __mbim_utils_get_devname (self->priv->path, &error);
+    if (!device_basename) {
+        g_warning ("[%s] Invalid path for cdc-wdm control port: %s",
+                   self->priv->path_display, error->message);
+        g_clear_error (&error);
+        goto out;
+    }
+
     device = g_udev_client_query_by_subsystem_and_name (client, "usb", device_basename);
     if (!device) {
         device = g_udev_client_query_by_subsystem_and_name (client, "usbmisc", device_basename);
